MODULE trdtra_oce
   !!======================================================================
   !!                   ***  MODULE trdtra_oce  ***
   !! Ocean trends :   set tracer trend variables
   !!======================================================================

   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_oce      ! ocean parameters

   IMPLICIT NONE
   PUBLIC

   INTEGER  ::      & !!! namdia :  diagnostics on dynamics and/or tracer trends
      ntrd  = 10 ,  &  !: time step frequency dynamics and tracers trends
      nctls =  0       !: control surface type for trends vertical integration
   !!----------------------------------------------------------------------

#if   defined key_trdtra   ||   defined key_trdmld   ||   defined key_esopa
   !!----------------------------------------------------------------------
   !!   'key_trdtra'   or                         tracer trends diagnostics
   !!   'key_trdmld'                         mixed layer trends diagnostics
   !!----------------------------------------------------------------------

   !! Trends diagnostics parameters
   !!---------------------------------------------------------------------
   INTEGER, PARAMETER ::            &
# if defined key_traldf_eiv
      jptrdh = 4,   &  !: number of 3D horiz trends arrays
      jpltrd = 10,  &  !: number of mixed-layer trends arrays
      jpktrd = jpk     !: max level for mixed-layer trends diag.
# else
      jptrdh = 2,   &  !: number of 3D horiz trends arrays
      jpltrd = 7,   &  !: number of mixed-layer trends arrays
      jpktrd = jpk     !: max level for mixed-layer trends diag.
# endif

   !! Trends diagnostics variables
   !!---------------------------------------------------------------------
   REAL(wp) ::   &
      tvolt         ! volume of the whole ocean computed at t-points
   REAL(wp), DIMENSION(jpi,jpj,jpk,7) ::   &
      ttrd             !: trends of the temperature tracer equations
      !                !  ttrd(,,,1) : horizontal advection
      !                !  ttrd(,,,2) : vertical advection
      !                !  ttrd(,,,3) : horizontal diffusion
      !                !  ttrd(,,,4) : vertical diffusion
      !                !  ttrd(,,,5) : static instability
      !                !  ttrd(,,,6) : damping OR vertical EIV
      !                !  ttrd(,,,7) : penetrative solar radiation (T only)
   REAL(wp), DIMENSION(jpi,jpj,jpk,6) ::   &
      strd             !: trends of the salinity tracer equations
      !                !  same as ttrd()
   REAL(wp), DIMENSION(jpi,jpj,jpk,jptrdh) ::   &
      ttrdh, strdh     !: ttrdh(,,,1) : zonal advection
      !                !  ttrdh(,,,2) : meridional advection
      !                !  ttrdh(,,,3) : zonal EIV
      !                !  ttrdh(,,,4) : meridional EIV
   REAL(wp), DIMENSION(jpi,jpj,2) ::   &
      flxtrd,       &  !: tracer forcing trends
      bbltrd           !: tracer bottom boundary layer trends
   
#else
  !!----------------------------------------------------------------------
  !!   Default option :                        NO tracer trend diagnostics
  !!----------------------------------------------------------------------
#endif

  !!======================================================================
END MODULE trdtra_oce
