MODULE oce
   !!======================================================================
   !!             ***  MODULE  oce  ***
   !! Ocean        :  dynamics and active tracers defined in memory 
   !!======================================================================
   !! History :
   !!   8.5  !  02-11  (G. Madec)  F90: Free form and module
   !!----------------------------------------------------------------------
   !!  OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_oce      ! ocean parameters

   IMPLICIT NONE

   !! Physics and algorithm flags
   !! ---------------------------
#if defined key_vectopt_memory
   LOGICAL ::   l_vopt   = .TRUE.   !: memory vector optimization flag
#else
   LOGICAL ::   l_vopt   = .FALSE.  !: memory vector optimization flag
#endif

   LOGICAL ::   ln_dynhpg_imp   = .FALSE.  ! semi-implicite hpg flag

   !! dynamics and tracer fields
   !! --------------------------
   REAL(wp), DIMENSION(jpi,jpj,jpk) ::   &
      ! before !  now      !  after  !
      ub       ,  un       ,  ua     ,   &  !: i-horizontal velocity (m/s)
      vb       ,  vn       ,  va     ,   &  !: j-horizontal velocity (m/s)
                  wn       ,             &  !: vertical velocity (m/s)
      rotb     ,  rotn     ,             &  !: relative vorticity (1/s)
      hdivb    ,  hdivn    ,             &  !: horizontal divergence (1/s)
      tb       ,  tn       ,  ta     ,   &  !: potential temperature (celcius)
      sb       ,  sn       ,  sa            !: salinity (psu)
   REAL(wp), DIMENSION(jpi,jpj,jpk) ::   &
      rhd ,                              &  !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
      rhop,                              &  !: potential volumic mass (kg/m3)
      rn2                                   !: brunt-vaisala frequency (1/s2)

   !! surface pressure gradient
   !! -------------------------
   REAL(wp), DIMENSION(jpi,jpj) ::   &
      spgu, spgv             !: horizontal surface pressure gradient

#if defined key_partial_steps     ||   defined key_esopa
   !! interpolated gradient
   !! ---------------------
   REAL(wp), DIMENSION(jpi,jpj) ::   &
      gtu, gsu, gru,      &  !: t-, s- and rd horizontal gradient at u- and 
      gtv, gsv, grv          !: v-points at bottom ocean level 
#else
   REAL(wp) ::   &
      gtu, gsu, gru,      &  !: dummy scalars
      gtv, gsv, grv          !:
#endif

#if defined key_dynspg_fsc   ||   defined key_esopa
   !! free surface - constant volume formulation
   !! ------------------------------------------
   REAL(wp), DIMENSION(jpi,jpj) ::   &
      sshb, sshn,         &  !: before, now sea surface height (meters)
      hu, hv                 !: depth at u- and v-points (meters)
# if defined key_obc
   REAL(wp) obcsurftot       !: Total lateral surface of open boundaries
   REAL(wp), DIMENSION(jpi,jpj) ::   &
      obcumask, obcvmask     !: u-, v- Force filtering mask for the open 
      !                      !  boundary condition on grad D
# endif
#endif
#if defined key_dynspg_rl   ||   defined key_esopa
   !! rigid-lid formulation
   !! ---------------------
   REAL(wp), DIMENSION(jpi,jpj) ::   &
      hur, hvr,           &  !: inverse of u and v-points ocean depth (1/m)
      bsfb, bsfn,         &  !: before, now barotropic streamfunction (m3/s)
      bsfd                   !: now trend of barotropic streamfunction (m3/s2)
#endif

   !!----------------------------------------------------------------------
END MODULE oce
